﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Syncfusion.Windows.Controls.Cells;
using Syncfusion.Windows.Controls.Grid;
using System.Collections.Specialized;
using System.Data;
using System.ComponentModel;
using System.Threading;

namespace DataTableSort
{
    class DataTableHelper
    {
        private GridControl grid;
        private DataView dataSource;
        

        public DataTableHelper(DataTable dataTable, GridControl grid)
        {
            this.grid = grid;

            grid.Model.CellModels.Add("MyHeader", new GridCellModel<MyHeaderRenderer>());
            MyHeaderRenderer renderer = grid.CellRenderers["MyHeader"] as MyHeaderRenderer;

             
            dataSource = dataTable.DefaultView;
            dataSource.ListChanged += new System.ComponentModel.ListChangedEventHandler(dataSource_ListChanged);
            renderer.SetGrid(grid, dataSource);

            grid.Model.RowCount = dataSource.Count + 1;
            grid.Model.ColumnCount = dataSource.Table.Columns.Count + 1;

            grid.ColumnWidths[0] = 25;

            //handle the virtual event
            grid.Model.QueryCellInfo += new GridQueryCellInfoEventHandler(Model_QueryCellInfo);
            this.grid.PreviewMouseLeftButtonDown += new MouseButtonEventHandler(grid_PreviewMouseLeftButtonDown);

            if (dataSource.AllowEdit)
            {
                this.grid.Model.CommitCellInfo += new GridCommitCellInfoEventHandler(Model_CommitCellInfo);
            }
              
        }

        void dataSource_ListChanged(object sender, System.ComponentModel.ListChangedEventArgs e)
        {
            if (e.ListChangedType == ListChangedType.Reset || e.ListChangedType == ListChangedType.ItemDeleted)
            {

                GridModel model = grid.Model;

                //model.SelectedRanges.Clear();
                grid.CurrentCell.Deactivate(true);
                model.VolatileCellStyles.Clear();
                grid.InvalidateCell(GridRangeInfo.Table());
                grid.InvalidateVisual();
                model.RowCount = dataSource.Count + 1;
            }
        }

        public void ApplyFilter(string filterString)
        {
                dataSource.RowFilter = filterString;
         }



        void Model_CommitCellInfo(object sender, GridCommitCellInfoEventArgs e)
        {
            if (e.Cell.ColumnIndex > 0 && e.Cell.RowIndex > 0)
            {
                try
                {
                    dataSource[e.Cell.RowIndex-1][e.Cell.ColumnIndex-1] = Convert.ChangeType(e.Style.CellValue, dataSource.Table.Columns[e.Cell.ColumnIndex - 1].DataType);
                    e.Handled = true;
                    grid.InvalidateCell(GridRangeInfo.Cell(e.Cell.RowIndex, e.Cell.ColumnIndex));
                    grid.InvalidateVisual(false);
                }
                catch
                {

                }
            }
        }


        void grid_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            GridControlBase grid = (GridControlBase)sender;
            RowColumnIndex cell = grid.PointToCellRowColumnIndex(e);
            if (cell.RowIndex == 0)
            {
                GridRenderStyleInfo style = grid.GetRenderStyleInfo(cell);
                string sort = dataSource.Sort.ToLower();
                string newSort = style.Description.ToLower() + " ";
                if (sort.StartsWith(newSort))
                {
                    if (sort.EndsWith("asc"))
                        newSort += "desc";
                    else
                        newSort += "asc";
                }
                else
                {
                    newSort += "asc";
                }
                dataSource.Sort = newSort;
                e.Handled = true;
                // grid.InvalidateCell(GridRangeInfo.Table());
                grid.InvalidateCell(GridRangeInfo.Cells(0, 0, grid.Model.RowCount, grid.Model.ColumnCount));
                grid.InvalidateVisual(false);
            }
        }

        void Model_QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
        {
            if (e.Cell.ColumnIndex > 0 && e.Cell.RowIndex == 0)
            {
                e.Style.Description = dataSource.Table.Columns[e.Cell.ColumnIndex - 1].ColumnName;
                //e.Style.CellType = "Button";
                e.Style.CellType = "MyHeader";
                e.Handled = true;
            }
            else if (e.Cell.RowIndex > 0 && e.Cell.ColumnIndex > 0)
            {
                e.Style.CellValue = dataSource[e.Cell.RowIndex - 1].Row[e.Cell.ColumnIndex - 1];
                e.Handled = true;
                //  Console.WriteLine(e.Cell);
            }
        }

    }

   

    public class MyHeaderRenderer : GridVirtualizingCellRenderer<StackPanel>
    {
        GridControl grid;
        DataView dataSource;
        BitmapImage up;
        BitmapImage down;
        private int arrowWidth = 0;
        TextBlock textBlock;
        Image c;

        public void SetGrid(GridControl grid, DataView dataSource)
        {
            this.grid = grid;
            this.dataSource = dataSource;
            Uri uri = new Uri("pack://application:,,/images/Down.ico");
            down = new BitmapImage(uri);
            uri = new Uri("pack://application:,,/images/upIcon.ico");
            up = new BitmapImage(uri);
            arrowWidth = 13;
        }

        protected override StackPanel CreateUIElement(ArrangeCellArgs aca, GridRenderStyleInfo cellInfo)
        {
            return base.CreateUIElement(aca, cellInfo);
        }

        public override void OnInitializeContent(StackPanel uiElement, GridRenderStyleInfo cellInfo)
        {
            // Console.WriteLine("OnInitializeContent");

            uiElement.Children.Clear();

            uiElement.Orientation = Orientation.Horizontal;

            textBlock = new TextBlock();
            textBlock.Text = cellInfo.Description;
            textBlock.TextAlignment = TextAlignment.Center;

            int colIndex = cellInfo.CellIdentity.CellRowColumnIndex.ColumnIndex;
            textBlock.Width = grid.ColumnWidths[colIndex] - arrowWidth; ;
            uiElement.Children.Add(textBlock);

            if (dataSource.Sort.ToLower().StartsWith(cellInfo.Description.ToLower() + " "))
            {
                // Image 
                c = new Image();
                if (dataSource.Sort.ToLower().EndsWith(" asc"))
                {
                    c.Source = up;
                }
                else //if (dataSource.Sort.ToLower().EndsWith(" desc"))
                {
                    c.Source = down;
                }
                uiElement.Children.Add(c);
            }

        }
    }
}
